(* code/switch.sml *)

functor SwitchFn(structure MakeString : MAKESTRING
		 structure ConRep : CONREP
		 structure Mangle : MANGLE
		   ) : SWITCH =
  struct

    structure ConRep	= ConRep
    structure Mangle	= Mangle

    datatype gvar_name	= SPgvn | FCgvn | SCgvn | ARGgvn of int
    datatype gvar_scope	= INTRAgvs | INTERgvs
    datatype gvar'	= GVARSTR of string | GVAR of {scope: gvar_scope, name: gvar_name}

    type gvar		= gvar'
    datatype lvar	= LVAR of int

    datatype variable	= GLOvar of gvar
			| LOCvar of lvar

    datatype label	= LABEL of Mangle.name

    datatype litname	= LITNAME of int
    datatype litref	= INTlr of int
			| HDRlr of {len: int, con: int}
			| LABELlr of label
			| EXTERNlr of label
			| REALlr of litname
			| STRINGlr of litname
			| STRUCTlr of litname
    datatype litdef	= REALld of real
			| STRINGld of string
			| STRUCTld of int * litref list

    datatype value	= VAR of variable
			| LITERAL of litref
			| OFFSET of value * int
			| FETCH of value
			| UNTAGPTR of value
			| TAGPTR of value
			| CALL of label * value list

    datatype casetag	= INTct of int
			| HDRct of {len: int, con: int}
			| REALct of real
			| STRINGct of string

    datatype gototarget = LOCALg of label
			| EXTERNg of label
			| VALUEg of value

    datatype code'	= GOTO of gototarget * int
			| STORE of value * value * code
			| BIND of variable option * value * code
			| SWITCH of value * (casetag * code) list * code option

    and code		= CODE of {fvars: lvar list ref, code: code'}

    datatype labdef	= LABDEF of {	globalP	: bool,
					label	: label,
					varHP	: lvar,
					nalloc	: int,
					nargs	: int,
					code	: code		}

    datatype module	= MODULE of {	modname	: string,
					ctors	: (string * ConRep.conrep) list,
					xmods	: string list,
					xlabs	: label list,
					xvals	: label list,
					values	: (label * litref) list,
					litdefs	: (litname * litdef) list,
					labdefs	: labdef list	}

    fun gvarString(GVAR{scope,name}) =
      let fun gvs2str(INTRAgvs) = "intra"
	    | gvs2str(INTERgvs) = "rml"
	  fun gvn2str(SPgvn) = "SP"
	    | gvn2str(FCgvn) = "FC"
	    | gvn2str(SCgvn) = "SC"
	    | gvn2str(ARGgvn i) = "A" ^ MakeString.icvt i
      in
	gvs2str scope ^ gvn2str name
      end

    fun lvarString(LVAR tag) = "tmp" ^ MakeString.icvt tag

    fun prGoto(os, prLabel, prValue, target, _) = (* nargs ignored *)
      case target
	of LOCALg label =>
	    (TextIO.output(os, "\n\tgoto label__");
	     prLabel os label;
	     TextIO.output(os, ";"))
	 | EXTERNg label =>
	    (TextIO.output(os, "\n\ttheLabel = &");
	     prLabel os label;
	     TextIO.output(os, ";\n\tgoto epilogue;"))
	 | VALUEg value =>
	    (TextIO.output(os, "\n\ttheLabel = (const rml_label_t*)(");
	     prValue os value;
	     TextIO.output(os, ");\n\tgoto mask_dispatch;"))

    fun mklab str = LABEL(Mangle.encode str)

    fun mkcode c = CODE{fvars=ref[], code=c}
    fun mkGOTO(target, nargs) = mkcode(GOTO(target, nargs))
    fun mkSTORE(v1,v2,c) = mkcode(STORE(v1,v2,c))
    fun mkBIND(xopt,v,c) = mkcode(BIND(xopt,v,c))
    fun mkSWITCH(v,cases,def) = mkcode(SWITCH(v,cases,def))

    val intraSP		= GLOvar(GVAR{scope=INTRAgvs, name=SPgvn})
    val intraFC		= GLOvar(GVAR{scope=INTRAgvs, name=FCgvn})
    val intraSC		= GLOvar(GVAR{scope=INTRAgvs, name=SCgvn})
    val intraArgs	=
      Vector.fromList[	GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 0}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 1}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 2}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 3}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 4}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 5}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 6}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 7}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 8}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 9}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 10}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 11}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 12}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 13}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 14}),
			GLOvar(GVAR{scope=INTRAgvs, name=ARGgvn 15})	]

    val interSP		= GLOvar(GVAR{scope=INTERgvs, name=SPgvn})
    val interFC		= GLOvar(GVAR{scope=INTERgvs, name=FCgvn})
    val interSC		= GLOvar(GVAR{scope=INTERgvs, name=SCgvn})
    val interArgs	=
      Vector.fromList[	GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 0}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 1}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 2}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 3}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 4}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 5}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 6}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 7}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 8}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 9}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 10}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 11}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 12}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 13}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 14}),
			GLOvar(GVAR{scope=INTERgvs, name=ARGgvn 15})	]

    (* These internal names are NOT mangled *)

    val primMARKER	= LABEL(Mangle.NAME "rml_prim_marker")
    val primUNWIND	= LABEL(Mangle.NAME "rml_prim_unwind")
    val primEQUAL	= LABEL(Mangle.NAME "rml_prim_equal")
    val primBOOL_NOT	= LABEL(Mangle.NAME "RML_PRIM_BOOL_NOT")
    val primINT_NEG	= LABEL(Mangle.NAME "RML_PRIM_INT_NEG")
    val primINT_ABS	= LABEL(Mangle.NAME "RML_PRIM_INT_ABS")
    val primBOOL_AND	= LABEL(Mangle.NAME "RML_PRIM_BOOL_AND")
    val primBOOL_OR	= LABEL(Mangle.NAME "RML_PRIM_BOOL_OR")
    val primINT_ADD	= LABEL(Mangle.NAME "RML_PRIM_INT_ADD")
    val primINT_SUB	= LABEL(Mangle.NAME "RML_PRIM_INT_SUB")
    val primINT_MUL	= LABEL(Mangle.NAME "RML_PRIM_INT_MUL")
    val primINT_DIV	= LABEL(Mangle.NAME "RML_PRIM_INT_DIV")
    val primINT_MOD	= LABEL(Mangle.NAME "RML_PRIM_INT_MOD")
    val primINT_MAX	= LABEL(Mangle.NAME "RML_PRIM_INT_MAX")
    val primINT_MIN	= LABEL(Mangle.NAME "RML_PRIM_INT_MIN")
    val primINT_LT	= LABEL(Mangle.NAME "RML_PRIM_INT_LT")
    val primINT_LE	= LABEL(Mangle.NAME "RML_PRIM_INT_LE")
    val primINT_EQ	= LABEL(Mangle.NAME "RML_PRIM_INT_EQ")
    val primINT_NE	= LABEL(Mangle.NAME "RML_PRIM_INT_NE")
    val primINT_GE	= LABEL(Mangle.NAME "RML_PRIM_INT_GE")
    val primINT_GT	= LABEL(Mangle.NAME "RML_PRIM_INT_GT")

  end (* functor SwitchFn *)
